/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.database.Cursor;
import android.util.Log;

import net.micode.notes.tool.GTaskStringUtils;

import org.json.JSONException;
import org.json.JSONObject;


public class MetaData extends Task {
    private final static String TAG = MetaData.class.getSimpleName();
    //相关GTask任务的ID
    private String mRelatedGid = null;

    //设置元数据,传入gid和元数据的JSONObject
    public void setMeta(String gid, JSONObject metaInfo) {
        //向元数据中加入相关GTaskID
        try {
            metaInfo.put(GTaskStringUtils.META_HEAD_GTASK_ID, gid);
        } catch (JSONException e) {
            Log.e(TAG, "failed to put related gid");
        }
        //设置笔记内容为元数据的JSON字符串
        setNotes(metaInfo.toString());
        //设置笔记名称
        setName(GTaskStringUtils.META_NOTE_NAME);
    }

    //获取相关GTask任务的ID
    public String getRelatedGid() {
        return mRelatedGid;
    }

    @Override
    public boolean isWorthSaving() {
        return getNotes() != null;
    }

    //从远程同步的数据中获取内容
    @Override
    public void setContentByRemoteJSON(JSONObject js) {
        //调用父类方法
        super.setContentByRemoteJSON(js);
        //如果有笔记内容
        if (getNotes() != null) {
            try {
                //解析笔记内容,获取相关GTask任务ID
                JSONObject metaInfo = new JSONObject(getNotes().trim());
                mRelatedGid = metaInfo.getString(GTaskStringUtils.META_HEAD_GTASK_ID);
            } catch (JSONException e) {
                Log.w(TAG, "failed to get related gid");
                mRelatedGid = null;
            }
        }
    }

    @Override
    public void setContentByLocalJSON(JSONObject js) {
        //此方法不应被调用,抛出异常
        throw new IllegalAccessError("MetaData:setContentByLocalJSON should not be called");
    }

    @Override
    public JSONObject getLocalJSONFromContent() {
        //此方法不应被调用,抛出异常
        throw new IllegalAccessError("MetaData:getLocalJSONFromContent should not be called");
    }

    @Override
    public int getSyncAction(Cursor c) {
        //此方法不应被调用,抛出异常
        throw new IllegalAccessError("MetaData:getSyncAction should not be called");
    }

}